#include "init.h"
#include "util.h"
#include "debug.h"

bool quitnow = false;
void Close() {
    DEBUGOUT("Received close message; quitnow variable set");
  quitnow = true;
}

int logic_time = 0;
void logic_timer()
{
  logic_time++;
}
END_OF_FUNCTION(logic_timer);

void SetFont(const char *fontfile) {
    FONT *newfont = GetFont(fontfile);
    if (!newfont) { // if failed to load pretty font, fall back to default allegro font
        DEBUGOUT("Loading font from " << fontfile << " failed, falling back on default");
        newfont = font;
    }
    DEBUGOUT("Setting global font");
    SetGlobalFont(newfont);
}

// initialize systems!
BITMAP* Init(int w, int h, const char *title) {
    DEBUGOUT("Initializing allegro.");
    allegro_init();
    DEBUGOUT("Installing mouse.");  
    install_mouse();
    DEBUGOUT("Installing timer.");
    install_timer();
    DEBUGOUT("Installing keyboard.");
    install_keyboard();

    DEBUGOUT("Setting graphics mode with 16 BPP");
    set_color_depth(16);
    if (set_gfx_mode(GFX_AUTODETECT_WINDOWED, w, h, 0, 0) != 0) {
        DEBUGOUT("16 BPP failed, trying 15");
        set_color_depth(15);
        if (set_gfx_mode(GFX_AUTODETECT_WINDOWED, w, h, 0, 0) != 0){
            DEBUGOUT("15 BPP also failed, trying full screen");
            if (set_gfx_mode(GFX_AUTODETECT, w, h, 0, 0) != 0) {
                set_color_depth(16);
                DEBUGOUT("15 BPP failed in full screen, trying 16 BPP full screen");
                if (set_gfx_mode(GFX_AUTODETECT, w, h, 0, 0) != 0) {
                    DEBUGOUT("Could not set graphics mode.");
                    allegro_message("Couldn't set graphics mode!");
                    exit(1);
                }
            }
        }
    }

    DEBUGOUT("Getting executable name, to set our path appropriately");
    char buf[1024];
    get_executable_name(buf, 1010);
    char *fname = (get_filename(buf));
    strcpy(fname, "/media/");
    chdir(buf);

    DEBUGOUT("Starting logic timer");
    LOCK_VARIABLE(logic_time);
    LOCK_FUNCTION(logic_timer);
    install_int_ex(logic_timer, BPS_TO_TIMER(60));

    DEBUGOUT("Setting close button callback");
    set_close_button_callback(Close);
    
    DEBUGOUT("Setting window title");
    set_window_title(title);

    return create_bitmap(w, h);
}

