#include <vector>

#include "util.h"
#include "input.h"
#include "debug.h"
#include "init.h"

using namespace std;

string GetWord(BITMAP *buffer, int x, int y, int maxletters, string prompt, string base) {
    DEBUGOUT("Trying to get a word from user, with prompt of " << prompt);
    
    string temp;
    char *name = (char*) malloc((maxletters + 1) * sizeof(char)); // ugh, rewrite to use std::string, soon.
    if (!name) {
        DEBUGOUT("Failed to allocate " << (maxletters + 1) * sizeof(char) << " bytes for word-to-input, can't read word.");
        return "malloc failed";
    }
    strncpy(name, base.c_str(), maxletters-1);
    name[maxletters-1] = '\0';
    int c = base.size() > (unsigned)maxletters - 1 ? (unsigned)maxletters - 1 : base.size();
    char ascii = 0;
    int height = text_height( Font() );
    
    // try to avoid having unrelated enter presses carry over, so it doesn't auto-accept
    while (key[KEY_ENTER] && !key[KEY_ESC] && !quitnow) {}
    clear_keybuf();
    
    while (!key[KEY_ENTER]) {
        if (key[KEY_ESC] || quitnow) {
            DEBUGOUT("Quit signal received in main GetWord loop, exit(0)ing");
            exit(0);
        }

        clear_to_color(buffer, makecol(255,255,255));
        textprintf_centre_ex(buffer, Font(), x, y, makecol(0,0,0), -1,  "%s", prompt.c_str());
        textprintf_centre_ex(buffer, Font(), x, y + height + 2, makecol(0,0,0), -1, "%s", name);
        blit(buffer, screen, 0, 0, 0, 0, SCREEN_W, SCREEN_H);

        while (!keypressed()) {
            if (quitnow) {
                DEBUGOUT("Quit signal received while waiting for keypress in GetWord, exit(0)ing");
                exit(0);
            }
        }

        int k = readkey();

        if (k >> 8 == KEY_BACKSPACE) {
        if (c > 0)
            c--;
            name[c] = '\0';
        }

        ascii = k & 0xff;
        if (c < maxletters && ((ascii >= 'A' && ascii <= 'Z') || ascii == ' ' || ascii == '.' || (ascii >= 'a' && ascii <= 'z') || (ascii >= '0' && ascii <= '9'))) {
            name[c] = ascii;
            name[++c] = '\0';
        }
        
        logic_time--;
		while (logic_time == 0)
			rest(0);
    }
    
    string ret = name;
    free(name);
    logic_time = 0;
    
    DEBUGOUT("Got word \"" << ret << "\" from prompt " << prompt);
    
    return ret;
}

std::string GetWord(BITMAP *buffer, std::string prompt, std::string base) {
    return GetWord(buffer, SCREEN_W/2, SCREEN_H/2, (SCREEN_W / text_length(Font(), "W")) - 2, prompt, base);
}

bool keyhit(char in) {
	static char array[256];
	static bool run = false;
	if (!run) {
		run = true;
		for (int i=0; i<256; i++) {
			array[i] = 0;
		}
	}
	if (!key[(int)in])
		array[(int)in] = false;
	else if (!array[(int)in]) {
		array[(int)in] = true;
		return true;
	}
	
	return false;
}


// helper function for SelectFile, below
int fill(const char *filename, int attrib, void* objectFiles) {
	((vector<string> *)objectFiles)->push_back(string(filename));
    DEBUGOUT("Saw file \"" << filename << " during file selection");
    return 0;
}

// selects a file
string SelectFile(BITMAP *buffer, const char *wildcard, const char *prompt) {
    DEBUGOUT("Trying to select a file");
    
	vector<string> objectFiles;
    //for_each_file_ex(const char *name, int in_attrib, int out_attrib, int (*callback)(const char *filename, int attrib, void *param), void *param);
    //FA_RDONLY | FA_ARCH
	for_each_file_ex(wildcard, 0, FA_DIREC, &fill, (void*)(&objectFiles));
    
	if (objectFiles.size() == 0) {
        DEBUGOUT("Tried to select a file with wildcard \"" << wildcard << "\" but there are no such files!");
        return "";
    }
    
    // try to avoid having unrelated enter presses carry over, so it doesn't auto-accept
    while (key[KEY_ENTER] && !key[KEY_ESC] && !quitnow) {}
    clear_keybuf();
    
    int height = text_height( Font() );
	unsigned int choice = 0;
	while (!key[KEY_ENTER]) {
        if (key[KEY_ESC] || quitnow) {
            DEBUGOUT("Quit signal received in main GetWord loop, exit(0)ing");
            exit(0);
        }
		
		if (keyhit(KEY_UP)) {
			if (choice == 0)
				choice = objectFiles.size() - 1;
			else
				choice--;
		}
		else if (keyhit(KEY_DOWN)) {
			choice = (choice + 1) % objectFiles.size();
		}
        
        clear_to_color(buffer, makecol(255,255,255));
		

        
        int offset = 0;
        if ((choice+4) * (height + 2) > (unsigned int)SCREEN_H)
            offset = SCREEN_H - (choice+4) * (height + 2);
		for (unsigned int i = 0; i < objectFiles.size(); i++) {
			if (i==choice) {
				rect(buffer, 10, (i+1) * (height + 2) + offset + 2, text_length( Font(), (objectFiles[i]+" ").c_str() ) + 2, (i+2) * (height + 2) - 1 + offset, makecol(0,255,0));
			}
            textprintf_ex(buffer, Font(), 15, (i+1) * (height + 2) + offset, makecol(0,0,0), -1,  "%s", objectFiles[i].c_str());
//			allegro_gl_printf(font, -2, 0 - (signed(i)-signed(choice))*factor, -5, makecol(255, 255, 255), "%s", objectFiles[i].c_str());
		}
        
        rectfill(buffer, 0, 0, SCREEN_W, 3 + height, makecol(255,255,255));
        textprintf_ex(buffer, Font(), 15, 2, makecol(0,0,0), -1,  "%s", prompt);

        blit(buffer, screen, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
		
		// logic is not seperate from drawing here, but we still want to manage speed so we don't clobber the system.
		logic_time--;
		while (logic_time == 0)
			rest(0);
	}
    
    DEBUGOUT("Selected file \"" << objectFiles[choice] << "\"");
	
	return objectFiles[choice];
}

bool Button::Inside(int x, int y) {
    return (x > upleft.x() && x < downright.x() && y > upleft.y() && y < downright.y());
}

std::string Button::GetName() {
    return name;
}

void Button::Draw(BITMAP *buffer, int x, int y) {
    if (Inside(x, y)) {
        rect(buffer, upleft.x(), upleft.y(),  downright.x(), downright.y(), makecol(0,255,0));
    } else {
        rect(buffer, upleft.x(), upleft.y(),  downright.x(), downright.y(), makecol(255,0,0));
    }
    rectfill(buffer, upleft.x() + 1, upleft.y() + 1,  downright.x() - 1, downright.y() - 1, makecol(255,255,255));
    textprintf_centre_ex(buffer, Font(), (upleft.x() + downright.x())/2, (upleft.y() + downright.y() - text_height(Font())) / 2, makecol(0,0,255), -1,  "%s", name.c_str());
}


void Buttons::AddButton(Button b) {
    buttons.push_back(b);
}

void Buttons::AddButton(Vec2i pos, string name) {
    AddButton(Button(pos, Vec2i(text_length(Font(), name.c_str()) + 10, text_height(Font())), name));
}

void Buttons::DrawButtons(BITMAP *buffer, int x, int y) {
    for (vector<Button>::iterator i = buttons.begin(); i != buttons.end(); i++) {
        i->Draw(buffer, x, y);
    }
}

string Buttons::GetButtonUnder(int x, int y) {
    for (vector<Button>::iterator i = buttons.begin(); i != buttons.end(); i++) {
        if (i->Inside(x,y)) {
            return i->GetName();
        }
    }
    
    return "";
}




