#include <allegro.h>
#include <cstring>
#include <cmath>
#include <iostream>
#include <fstream>
#include <list>

#include "init.h"
#include "image.h"
#include "util.h"
#include "debug.h"
#include "input.h"
#include "vec2.h"

using namespace std;

#define RADBIG 188
#define RADSM 95
#define MAX_VEL 20
#define MAX_ANG_VEL .3
#define MAX_COMPRESS .5
#define JUMP 23
#define AIRBORN_THRESH 2
#define WALL_WIDTH 20

#define DEBUGGFX false

double max(double a, double b) {
    return a > b ? a : b;
}

double min(double a, double b) {
    return a < b ? a : b;
}

struct Fly {
    Vec2d pos;
    Vec2d vel;
    Image *frameOne;
    Image *frameTwo;
    
    Fly(Vec2d pos, Image *f1, Image *f2) : pos(pos), vel(0,0), frameOne(f1), frameTwo(f2) {}
    
    void flyRandom(int gheight) {
        
        // randomly move
        int maxvel = 8;
        vel += Vec2d(rand() % 3 - 1, rand() %3 - 1);
        vel = Vec2d(vel.x() > maxvel ? maxvel : vel.x(),
                          vel.y() > maxvel ? maxvel : vel.y());
        
        pos += vel + Vec2d(rand() % 5 - 2, rand() % 5 -2);
        
        // keep the fly 'in bounds'
        if (pos.x() < WALL_WIDTH) {
            pos = Vec2d(WALL_WIDTH, pos.y());
            vel = Vec2d(-vel.x(), vel.y());
        } else if (pos.x() > SCREEN_W - WALL_WIDTH - frameOne->bitmap->w) {
            pos = Vec2d(SCREEN_W - WALL_WIDTH - frameOne->bitmap->w, pos.y());
            vel = Vec2d(-vel.x(), vel.y());
        }
        if (pos.y() > gheight - frameOne->bitmap->h - 200) {
            pos = Vec2d(pos.x(), gheight - frameOne->bitmap->h - 200);
            vel = Vec2d(vel.x(), -vel.y());
        } else if (pos.y() < 0) {
            pos = Vec2d(pos.x(), 0);
            vel = Vec2d(vel.x(), -vel.y());
        }
    }
    
    void Draw(BITMAP *buffer) {
        BITMAP *toDraw = frameOne->bitmap;
        if ((rand()) % 4 == 1) {
            toDraw = frameTwo->bitmap;
        }
        
        draw_sprite(buffer, toDraw, int(pos.x()), int(pos.y()));
    }
};

enum Mode {FLYING, ROLLING, STICKING, STUCK};

struct Nana {
    Image *img;
    Vec2d center;
    float angle;
    Vec2d vel;
    float angleVel;
    Mode lastMode;
    float compression;
    BITMAP *flipped;
    
    bool hitSideFlag;
    
    Nana(Image *img, Vec2d center, float angle) : img(img), center(center), angle(angle), vel(0,0), lastMode(FLYING), compression(1) {
        flipped = create_bitmap(img->bitmap->w, img->bitmap->h);
        clear_to_color(flipped, makecol(255,0,255));
        if (!flipped) {
            DEBUGOUT("Couldn't create buffer for 180 rot'd copy of sprite");
        }
        rotate_sprite(flipped, img->bitmap, 0, 0, itofix(128));
    }
    
    Vec2d cBigCen() {
        return Vec2d(center.x() + 129*sin(angle), center.y() - 129 * cos(angle));
    }
    
    bool Collides(Vec2d pos) {
        
        Vec2d d1 = pos - center;
        Vec2d d2 = pos - cBigCen();
        int leeway = 10;
        return (d1.magnitudeSquared() < (RADSM + leeway)*(RADSM + leeway) && d2.magnitudeSquared() > (RADBIG - leeway)*(RADBIG - leeway));
    }
    
    Mode DoPhysics(int push, bool jump, int gheight) {
        angleVel += push * .003;
        if (angleVel > MAX_ANG_VEL) {
            angleVel = MAX_ANG_VEL;
        } else if (angleVel < -MAX_ANG_VEL) {
            angleVel = -MAX_ANG_VEL;
        }
        
        angleVel *= .99;
        
        {
            int dir = 0;
            if (hitSide(WALL_WIDTH, 1)) { // right wall
                dir = 1;
            } else if (hitSide(WALL_WIDTH, -1)) { // left wall
                dir = -1;
            }
            
            hitSideFlag = false;
            if (dir != 0) {
                hitSideFlag = true;
                if (angleVel * dir > 0)
                    angleVel = -1.2*angleVel;
                if (vel.x() * dir > 0)
                    vel = Vec2d(-1.1*vel.x(), vel.y());
                center = Vec2d(center.x() - lastPenEst, center.y());
            }
            
        }
        
        
        if (!hitGround(gheight)) {
            if (lastMode == ROLLING) {
                vel = Vec2d(angleVel * RADSM, vel.y()); // 1.3 for the extra bit of cool!
            }
            if (vel.y() < MAX_VEL) {
                vel += Vec2d(0,.4); // gravity
            }
            center += vel;
            Vec2d spcen = center - Vec2d(68*sin(angle), -68*cos(angle));
            angle+=angleVel;
            Vec2d spcen2 = center - Vec2d(68*sin(angle), -68*cos(angle));
            center += spcen - spcen2;
        }
        
        Mode ourmode;
        if (!hitGround(gheight)) {
            ourmode = FLYING;
        } else {
            vel *= .99;
            Vec2d normal(sin(angle), -cos(angle));
            if (normal.y() > -.2) {
                bool once = false;
            retry:
                ourmode = STICKING;
                Vec2d delta1(sin(-angle) * img->bitmap->h/2, cos(-angle) * img->bitmap->h/2);
                Vec2d delta2(sin(-angle + PI/2.0) * img->bitmap->w/2, cos(-angle + PI/2.0) * img->bitmap->w/2);
                Vec2d spcen = center - Vec2d(68*sin(angle), -68*cos(angle));
                Vec2d corner1 = spcen + delta1 + delta2, corner2 = spcen + delta1 - delta2;
                Vec2d corner3 = spcen - delta1 - delta2, corner4 = spcen - delta1 + delta2;
                Vec2d stuck = corner4, unstuck = corner3;
                int angleDir = 1;
                if (corner3.y() > corner4.y()) {
                    stuck = corner3;
                    unstuck = corner4;
                    angleDir = -1;
                }
                
                if (unstuck.y() > gheight || lastMode == STUCK) {
                    ourmode = STUCK;
                    
                    if (lastMode != STUCK) {
                        {
                            vel = Vec2d(vel.x(), 0);
                        }
                        
                        angleVel = 0;
                        angle = PI;
                        center = Vec2d(center.x(), gheight + RADSM - img->bitmap->h + 3);
                    }
                    
                   // vel *= .98; // friction
                    if (vel.x() < 1 && vel.x() > -1)
                        vel = Vec2d(0,0);
                    center += vel;
                    
                    if (jump) {
                        if (compression > MAX_COMPRESS)
                            compression = MAX_COMPRESS + (compression - MAX_COMPRESS) * .96;
                    } else if (compression < 1) {
                        vel = Vec2d(vel.x(), - JUMP * (1.0-compression) / (1.0-MAX_COMPRESS));
                        center += vel;
                        ourmode = FLYING;
                        compression = 1;
                    }
                } else {
                    if (once) {
                        goto done;
                    }
                    
                    Vec2d spcenFirst = center - Vec2d(30*sin(angle), -10*cos(angle));
                    
                    angleVel += angleDir * .0035 * (normal.y() < .1 ? -1 : 1);
                    angle += angleVel;
                    
                    Vec2d spcenSecond = center - Vec2d(30*sin(angle), -10*cos(angle));
                    center += spcenFirst - spcenSecond;
                    
                    
                    
                    Vec2d delta1(sin(-angle) * img->bitmap->h/2, cos(-angle) * img->bitmap->h/2);
                    Vec2d delta2(sin(-angle + PI/2.0) * img->bitmap->w/2, cos(-angle + PI/2.0) * img->bitmap->w/2);
                    Vec2d spcen = center - Vec2d(68*sin(angle), -68*cos(angle));
                    Vec2d newStuck = spcen - delta1 + double(angleDir) * delta2;
                    
                    if (newStuck.y() < gheight - AIRBORN_THRESH) {
                        vel = Vec2d(angleVel * RADSM, 0);
                        vel += Vec2d(0,.4); // gravity
                        center += vel;
                        ourmode = FLYING;
                    } else {
                        center += Vec2d((stuck - newStuck).x(), 0); // move stuck back to orig pos -- it's stuck!
                        spcen = center - Vec2d(68*sin(angle), -68*cos(angle));
                        newStuck = spcen - delta1 + double(angleDir) * delta2;
                        if (newStuck.y() > gheight - 1) {
                            center -= Vec2d(0, newStuck.y() - (gheight - 1));
                        }
                        once = true;
                        
                        goto retry;
                    }
                }
            } else {
                if (lastMode == FLYING) {
                    angleVel += (vel.x() / RADSM) * .25;
                }
                angleVel -= sin(angle) * .02;
                //angleVel *= .99;
                angle+=angleVel;
                center = Vec2d(center.x() + angleVel * RADSM, gheight-RADSM+1);
                ourmode = ROLLING;
            }
        }
    done:
        lastMode = ourmode;
        return ourmode;
    }
    
    int hitGround(int gheight) {
        if (center.y() + RADSM > gheight) {
            Vec2d delta1(sin(-angle) * img->bitmap->h/2, cos(-angle) * img->bitmap->h/2);
            Vec2d delta2(sin(-angle + PI/2.0) * img->bitmap->w/2, cos(-angle + PI/2.0) * img->bitmap->w/2);
            Vec2d spcen = center - Vec2d(68*sin(angle), -68*cos(angle));
            Vec2d corner1 = spcen + delta1 + delta2, corner2 = spcen + delta1 - delta2;
            Vec2d corner3 = spcen - delta1 - delta2, corner4 = spcen - delta1 + delta2;
            return (max(max(corner1.y(),corner2.y()), max(corner3.y(), corner4.y())) > gheight);
        }
        
        return false;
    }
    
    float lastPenEst;
    
    bool hitSide(int swidth, int dir) {
        //DEBUGOUT("Compare " << (dir*(center.x() + dir*RADSM)) << " with " << ((dir > 0 ? SCREEN_W - swidth : -swidth)) << " on swidth " << swidth);
        if (dir*(center.x() + dir*RADSM) > (dir > 0 ? SCREEN_W - swidth : -swidth)) {
            Vec2d delta1(sin(-angle) * img->bitmap->h/2, cos(-angle) * img->bitmap->h/2);
            Vec2d delta2(sin(-angle + PI/2.0) * img->bitmap->w/2, cos(-angle + PI/2.0) * img->bitmap->w/2);
            Vec2d spcen = center - Vec2d(68*sin(angle), -68*cos(angle));
            Vec2d corner1 = spcen + delta1 + delta2, corner2 = spcen + delta1 - delta2;
            Vec2d corner3 = spcen - delta1 - delta2, corner4 = spcen - delta1 + delta2;
            if (dir > 0) {
                double est = max(max(corner1.x(),corner2.x()), max(corner3.x(), corner4.x()));
                if (est > SCREEN_W - swidth) {
                    double est2 = center.x() + RADSM - (SCREEN_W - swidth);
                    lastPenEst = est - (SCREEN_W - swidth);
                    if (fabs(est2) < fabs(lastPenEst))
                        lastPenEst = est2;
                    return true;
                }
            } else {
                double est = (min(min(corner1.x(),corner2.x()), min(corner3.x(), corner4.x())) );
                if (est < swidth) {
                    double est2 = (center.x() - RADSM) - swidth;
                    lastPenEst = est - swidth;
                    if (fabs(est2) < fabs(lastPenEst))
                        lastPenEst = est2;
                    return true;
                }
            }
        }
        
        return false;
    }
    
    
    void Draw(BITMAP *buffer) {
        Vec2d delta1(sin(-angle) * img->bitmap->h/2, cos(-angle) * img->bitmap->h/2);
        Vec2d delta2(sin(-angle + PI/2.0) * img->bitmap->w/2, cos(-angle + PI/2.0) * img->bitmap->w/2);
        Vec2d spcen = center - Vec2d(68*sin(angle), -68*cos(angle));
        Vec2d corner1 = spcen + delta1 + delta2, corner2 = spcen + delta1 - delta2;
        Vec2d corner3 = spcen - delta1 - delta2, corner4 = spcen - delta1 + delta2;
        
        if (lastMode == STUCK) { //+ (1-compression)*img->bitmap->w/2) //+ (1-compression) * img->bitmap->h
            masked_stretch_blit(flipped, buffer, 0, 0, img->bitmap->w, img->bitmap->h, (int)corner1.x(), int(corner1.y() + (1-compression) * img->bitmap->h), img->bitmap->w, int(img->bitmap->h * compression));
        } else {
            rotate_sprite(buffer, img->bitmap, int(center.x() - 68*sin(angle) - img->bitmap->w/2), int(center.y() + 68 * cos(angle) - img->bitmap->h/2), itofix((int)(angle * 128/PI)));
        }
        
        if (DEBUGGFX) {
            circle(buffer, int(cBigCen().x()), int(cBigCen().y()), RADBIG, makecol(0,255,0));
            circle(buffer, int(center.x()), int(center.y()), RADSM, makecol(0,255,0));

            line(buffer, int(corner1.x()), int(corner1.y()), int(corner2.x()), int(corner2.y()), makecol(255,0,0));
            line(buffer, int(corner2.x()), int(corner2.y()), int(corner3.x()), int(corner3.y()), makecol(0,255,0));
            line(buffer, int(corner3.x()), int(corner3.y()), int(corner4.x()), int(corner4.y()), makecol(0,0,255));
            line(buffer, int(corner4.x()), int(corner4.y()), int(corner1.x()), int(corner1.y()), makecol(0,255,255));
            line(buffer, int(center.x()), int(center.y()), int(center.x() + sin(angle) * RADSM), int(center.y() - cos(angle) * RADSM), makecol(255,255,0));
            circle(buffer, int(corner3.x()), int(corner3.y()), 10, makecol(255,0,0));
            circle(buffer, int(corner4.x()), int(corner4.y()), 10, makecol(0,0,255));
        }
    }
    
};

void Splatter(BITMAP *img, Vec2d center) {
    int splatterparts = 150;
    for (int i = 0; i < splatterparts; i++) {
        int rad = rand() % 10 + rand() % 10 + rand() % 10;
        int ang = rand() % 360;
        double angle = ang * PI/180.0;
        putpixel(img, int(center.x() + cos(angle) * rad), int(center.y() + sin(angle) * rad), makecol(rand() % 100, rand() % 100+155,0));
    }
}



int main() {

    BITMAP *buffer = Init(1024,768, "demo");
    if (!buffer) {
        DEBUGOUT("Initialization failed -- invalid backbuffer on exit");
        exit(1);
    }
    DEBUGOUT("Initialization succeeded");
    
    SetFont("font.dat");

    DEBUGOUT("Beginning game loop");
    
    Buttons main;
    main.AddButton(Vec2i(SCREEN_W/2 - text_length(Font(), "Single Player") / 2,SCREEN_H - 200), "Single Player");
    main.AddButton(Vec2i(SCREEN_W/2 - text_length(Font(), "Two Player") / 2,SCREEN_H - 150), "Two Player");
    main.AddButton(Vec2i(SCREEN_W/2 - text_length(Font(), "Help") / 2,SCREEN_H - 100), "Help");
    main.AddButton(Vec2i(SCREEN_W/2 - text_length(Font(), "Quit") / 2,SCREEN_H - 50), "Quit");
    
    Buttons help1;
    help1.AddButton(Vec2i(100 - text_length(Font(), "Back") / 2,SCREEN_H - 50), "Back");
    help1.AddButton(Vec2i(SCREEN_W - 100 - text_length(Font(), "Next") / 2,SCREEN_H - 50), "Next");
    
    Buttons help2;
    help2.AddButton(Vec2i(100 - text_length(Font(), "Back") / 2,SCREEN_H - 50), "Back");
    help2.AddButton(Vec2i(SCREEN_W - 100 - text_length(Font(), "Done") / 2,SCREEN_H - 50), "Done");
    
    Buttons *currentInterface = &main;
    
    ImageManager images;
    Image *nan = images.GetImage("banana_sm1.bmp");
    Image *title = images.GetImage("title800_3.bmp");
    Image *nan2 = images.GetImage("banana_sm1.bmp");
    Image *f1 = images.GetImage("fly1.bmp");
    Image *f2 = images.GetImage("fly2.bmp");
    Image *helpImg = images.GetImage("help1.bmp");
    Image *helpImg2 = images.GetImage("help2.bmp");
    
    
    BITMAP *background = create_bitmap(SCREEN_W, SCREEN_H);

    
    while (!key[KEY_ESC] && !quitnow){
        
        logic_time = 0;
        int p2exists = false;
        BITMAP *screeny = title->bitmap;
        bool mouseBeenDown = false;
        
        while (!key[KEY_ESC] && !quitnow) {
            
            while (logic_time > 0) {
                logic_time--;
            }
            
            if (mouse_b & 1) {
                mouseBeenDown = true;
            }
            
            if (!mouse_b&1) {
                if (mouseBeenDown) {
                    string pressedName = currentInterface->GetButtonUnder(mouse_x, mouse_y);
                    if (pressedName != "") {
                        if (pressedName == "Single Player") {
                            break;
                        } else if (pressedName == "Two Player") {
                            p2exists = true;
                            break;
                        } else if (pressedName == "Help") {
                            screeny = helpImg->bitmap;
                            currentInterface = &help1;
                        } else if (pressedName == "Back") {
                            if (currentInterface == &help1) {
                                screeny = title->bitmap;
                                currentInterface = &main;
                            } else {
                                screeny = helpImg->bitmap;
                                currentInterface = &help1;
                            }
                        } else if (pressedName == "Next") {
                            screeny = helpImg2->bitmap;
                            currentInterface = &help2;
                        } else if (pressedName == "Done") {
                            screeny = title->bitmap;
                            currentInterface = &main;
                        } else if (pressedName == "Quit") {
                            DEBUGOUT("Quitting game from main menu.");
                            quitnow = true;
                        } else {
                            allegro_message("Functionality for %s not yet implemented", pressedName.c_str());
                        }
                    }
                }
                mouseBeenDown = false;
            }
            
            blit(screeny, buffer, 0, 0, (SCREEN_W-screeny->w)/2, (SCREEN_H-screeny->h)/2, screeny->w, screeny->h);
            ///blit(title->bitmap, buffer, 0, 0, (SCREEN_W-title->bitmap->w)/2, (SCREEN_H-title->bitmap->h)/2, title->bitmap->w, title->bitmap->h);
            
            currentInterface->DrawButtons(buffer, mouse_x, mouse_y);
            rect(buffer, mouse_x, mouse_y, mouse_x + 5, mouse_y + 5, makecol(255,0,0));
            
                
            blit(buffer, screen, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
            clear_to_color(buffer, makecol(255,255,255));
        }
        
        
        int score1 = 0, score2 = 0;
        int level = 1;
        
        while (!key[KEY_ESC] && !quitnow) { // level loop
            // preloading for a level
            logic_time = 0;
    //        float angleSpd = .1;
            Mode m = FLYING;
            Nana theNan(nan, Vec2d(p2exists ? SCREEN_W/4 : SCREEN_W/2, SCREEN_H - 100), 0);
            Nana theOtherNan(nan2, Vec2d(3*SCREEN_W/4, SCREEN_H - 100), 0);
            list<Nana> p1ExtraNans, p2ExtraNans;
            clear_to_color(background, makecol(255,255,255));
            
            
            int numFlies = min(5 * level, 80);
            list<Fly> flies;
            for (int i = 0; i < numFlies; i++) {
                flies.push_back(Fly(Vec2d(rand()%SCREEN_W, rand()%SCREEN_H), f1, f2));
            }
            
            clear_to_color(buffer, makecol(255,255,255));
            textprintf_centre_ex(buffer, Font(), SCREEN_W/2, SCREEN_H/2, makecol(0,0,0), -1,  "Level %d", level);
            textprintf_centre_ex(buffer, Font(), SCREEN_W/2, SCREEN_H/2 + 40, makecol(0,0,0), -1,  "Press any key to begin.");
            blit(buffer, screen, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
            clear_keybuf();
            readkey();
            
            bool won = false;
            int killInAirP1= 0, killInAirP2 = 0;
            int p1Ultra = 0, p2Ultra = 0;
            int timeLeft = 60 * 60 - 5 * (level < 8 ? level : 8) * 60;
            // main game
            while (!key[KEY_ESC] && !quitnow && !won) {
                if (flies.empty()) {
                    won = true;
                } else if (timeLeft < 0) {
                    break;
                }
                
                while (logic_time > 0) {
                    timeLeft--;
                    p1Ultra--;
                    p2Ultra--;
                    
                    if (p1Ultra <= 0) {
                        p1ExtraNans.clear();
                    }
                    if (p2Ultra <= 0) {
                        p2ExtraNans.clear();
                    }
                    
                    m = theNan.DoPhysics((key[KEY_RIGHT]||key[KEY_D]) - (key[KEY_A]||key[KEY_LEFT]), key[KEY_SPACE] || key[KEY_DOWN] || key[KEY_S], SCREEN_H - 20);
                    for (list<Nana>::iterator i = p1ExtraNans.begin(); i != p1ExtraNans.end(); ++i) {
                        i->DoPhysics((key[KEY_RIGHT]||key[KEY_D]) - (key[KEY_A]||key[KEY_LEFT]), key[KEY_SPACE] || key[KEY_DOWN] || key[KEY_S], SCREEN_H - 20);
                    }
                    int ultraLen = 60 * 10;
                    int extraNans = 3;
                    if (m != FLYING) {
                        if (killInAirP1 > 3 && p1Ultra <= -3*60) {
                            p1Ultra = ultraLen; // five seconds of ultrakill mode
                            p1ExtraNans.clear();
                            for (int nans = 0; nans < extraNans + killInAirP1/3; nans++) {
                                Nana recent(nan, theNan.center + Vec2d(rand()%201-100, -(rand() % 30)), (rand() % 360) * (PI/360));
                                recent.vel = theNan.vel + Vec2d(rand()%11-5, -(rand() % 5));
                                recent.angleVel = theNan.angleVel + (rand()%5) * (PI / 360);
                                p1ExtraNans.push_back(recent);
                            }
                        }
                        killInAirP1 = 0;
                    }
                    if (p2exists) {
                        Mode m2 = theOtherNan.DoPhysics( ((mouse_b & 2) && 1) - ((mouse_b & 1) && 1), (mouse_b & 1) && (mouse_b & 2), SCREEN_H - 20);
                        for (list<Nana>::iterator i = p2ExtraNans.begin(); i != p2ExtraNans.end(); ++i) {
                            i->DoPhysics( ((mouse_b & 2) && 1) - ((mouse_b & 1) && 1), (mouse_b & 1) && (mouse_b & 2), SCREEN_H - 20);
                        }
                        if (m2 != FLYING) {
                            if (killInAirP2 > 3 && p2Ultra <= -3*60) {
                                p2Ultra = ultraLen;
                                p2ExtraNans.clear();
                                for (int nans = 0; nans < extraNans + killInAirP2/3; nans++) {
                                    Nana recent(nan2, theOtherNan.center + Vec2d(rand()%201-100, -(rand() % 30)), (rand() % 360) * (PI/360));
                                    recent.vel = theOtherNan.vel + Vec2d(rand()%3-1, -(rand() % 3));
                                    recent.angleVel = theOtherNan.angleVel + (rand()%5) * (PI / 360);
                                    p2ExtraNans.push_back(recent);
                                }
                            }
                            killInAirP2 = 0;
                        }
                    }
                    for (list<Fly>::iterator i = flies.begin(); i != flies.end(); i++) {
                        i->flyRandom(SCREEN_H - 20);
                        bool p1Kill = theNan.Collides(i->pos + Vec2d(21,19));
                        for (list<Nana>::iterator j = p1ExtraNans.begin(); j != p1ExtraNans.end(); ++j) {
                            if (p1Kill) {
                                break;
                            }
                            p1Kill = j->Collides(i->pos + Vec2d(21,19));
                        }
                        score1 += p1Kill * level;
                        killInAirP1 += p1Kill;
                        
                        bool p2Kill = false;
                        if (p2exists) {
                            p2Kill = theOtherNan.Collides(i->pos + Vec2d(21,19));
                            for (list<Nana>::iterator j = p2ExtraNans.begin(); j != p2ExtraNans.end(); ++j) {
                                if (p2Kill) {
                                    break;
                                }
                                p2Kill = j->Collides(i->pos + Vec2d(21,19));
                            }
                            killInAirP2 += p2Kill;
                            score2 += p2Kill * level; // system allows simultaneous kills of the same fly
                        }
                        if (p1Kill || p2Kill) {
                            Splatter(background, i->pos);
                            i = flies.erase(i);
                        }
                    }
                    logic_time--;
                }
                
                //vsync();
                if (theNan.hitGround(SCREEN_H-20) && DEBUGGFX) {
                    clear_to_color(buffer, makecol(255,150,150));
                }
                for (list<Fly>::iterator i = flies.begin(); i != flies.end(); i++) {
                    i->Draw(buffer);
                }
                theNan.Draw(buffer);
                //if (p1Ultra > 60*2 || logic_time % 20 < 10) {
                    for (list<Nana>::iterator i = p1ExtraNans.begin(); i != p1ExtraNans.end(); ++i) {
                        i->Draw(buffer);
                    }
                //}
                textprintf_ex(buffer, Font(), SCREEN_W - 300, 50, makecol(0,0,0), -1,  "P1: %d", score1);
                textprintf_centre_ex(buffer, Font(), SCREEN_W/2, 50, makecol(0,0,0), -1,  "Time Left: %d", timeLeft / 60);
                if (p1Ultra > 0) {
                    textprintf_centre_ex(buffer, Font(), p2exists ? SCREEN_W/4 : SCREEN_W/2 + rand() % 7 - 3, SCREEN_H/2 + rand() % 7 - 3, makecol(255,0,0), -1,  "CRAZY");
                }
                if (p2exists) {
                    theOtherNan.Draw(buffer);
                    //if (p2Ultra > 60*2 || logic_time % 20 < 10) {
                        for (list<Nana>::iterator i = p2ExtraNans.begin(); i != p2ExtraNans.end(); ++i) {
                            i->Draw(buffer);
                        }
                    //}
                    textprintf_ex(buffer, Font(), SCREEN_W - 300, 100, makecol(0,0,0), -1,  "P2: %d", score2);
                    if (p2Ultra > 0) {
                        textprintf_centre_ex(buffer, Font(), 3*SCREEN_W/4 + rand() % 7 - 3, SCREEN_H/2 + rand() % 7 - 3, makecol(255,0,0), -1,  "CRAZY");
                    }
                }
                
                {
                    rectfill(buffer, 0, SCREEN_H-21, SCREEN_W, SCREEN_H, makecol(0,0,0));
                    rectfill(buffer, 0, 0, WALL_WIDTH, SCREEN_H, makecol(0,0,0));
                    rectfill(buffer, SCREEN_W-WALL_WIDTH, 0, SCREEN_W, SCREEN_H, makecol(0,0,0));
                    if (theNan.hitSideFlag && DEBUGGFX) {
                        rectfill(buffer, 0, 0, WALL_WIDTH, SCREEN_H, makecol(255,0,0));
                        rectfill(buffer, SCREEN_W-WALL_WIDTH, 0, SCREEN_W, SCREEN_H, makecol(255,0,0));
                    }
                    if (m == ROLLING && DEBUGGFX) {
                        rectfill(buffer, 0, SCREEN_H-20, SCREEN_W, SCREEN_H, makecol(100,255,100));
                    }
                }

                blit(buffer, screen, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
                clear_to_color(buffer, makecol(255,255,255));
                while (logic_time == 0) {
                    rest(0);
                }
                blit(background, buffer, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
            }
            
            if (won) {
                clear_to_color(buffer, makecol(255,255,255));
                textprintf_centre_ex(buffer, Font(), SCREEN_W/2, SCREEN_H/2, makecol(0,0,0), -1,  "Level %d clear!", level++);
                textprintf_centre_ex(buffer, Font(), SCREEN_W/2, SCREEN_H/2 + 40, makecol(0,0,0), -1,  "Press any key to continue.");
                blit(buffer, screen, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
                clear_keybuf();
                readkey();
            } else if (timeLeft < 0) {
                clear_to_color(buffer, makecol(255,255,255));
                textprintf_centre_ex(buffer, Font(), SCREEN_W/2, SCREEN_H/2, makecol(0,0,0), -1,  "Time up!  The fruit flies got to you ...");
                textprintf_centre_ex(buffer, Font(), SCREEN_W/2, SCREEN_H/2 + 40, makecol(0,0,0), -1,  "Press any key to continue.");
                blit(buffer, screen, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
                clear_keybuf();
                readkey();
                
                HighScores(buffer, score1, "Player 1");
                if (p2exists) {
                    HighScores(buffer, score2, "Player 2");
                }
                break;
            }
        }
    }
    //cleanup
    return 0;
}END_OF_MAIN()


