#include "util.h"
#include "init.h"
#include "debug.h"

#include <list>
using namespace std;

BITMAP *LoadBitmap(const char * file, RGB *palette) {
    BITMAP *tr = load_bitmap(file, palette);
    if (!tr) {
        DEBUGOUT("Couldn't load bitmap: " << file);
        exit(1);
    }
    return tr;
}

void DoSplash(const char *file, BITMAP *buffer) {
    if (!buffer) {
        DEBUGOUT("Failing to make splash screen -- need a back buffer");
        exit(1);
    }
    
    BITMAP *splash = LoadBitmap(file);
    
    DEBUGOUT("Ready to display splash screen");
    
    while (key[KEY_ENTER]) {}
        clear_keybuf();
    while (!key[KEY_ENTER]) {
        if (key[KEY_ESC] || quitnow)
            exit(0);

        blit(splash, buffer, 0, 0, (buffer->w - splash->w)/2, (buffer->h - splash->h)/2, SCREEN_W, SCREEN_H);
        if (logic_time > 60)
            textprintf_centre_ex(buffer, Font(), SCREEN_W/2, SCREEN_H-40, makecol(0,0,0), -1,  "Press [Enter] to continue");
        blit(buffer, screen, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
    }
    
    DEBUGOUT("Leaving splash screen");
    destroy_bitmap(splash);
    splash = NULL;
}

FONT *GetFont(const char *fontfile) {
    DEBUGOUT("Loading font " << fontfile);
    
    DATAFILE* fontdat = load_datafile(fontfile);
    if (!fontdat) {
        DEBUGOUT("Failed to load font data file " << fontfile); 
        return NULL;
    }
    
    FONT* newfont = (FONT*)fontdat[0].dat;
    if (!newfont) {
        DEBUGOUT("Failed to load font as first item in data file " << fontfile);
        return NULL;
    }
    
    return newfont;
}

FONT *globalFont = font;

FONT *Font() {
    return globalFont;
}

void SetGlobalFont(FONT *f) {
    if (f)
        globalFont = f;
}




struct Score {
  Score(string name, int score) : name(name), score(score) {}
  Score() : score(0) {}
  
  string name;
  int score;
};


#define MAX_SCORES 10

void HighScores(BITMAP *buffer, int score, const char* who) {
  list<Score> scores;
  
  ifstream file("scores");
  string temp;
  int worst = -1;
  int count = 0;
  int rank = -1;
  char c;
  while (count < MAX_SCORES && getline(file, temp)) {
    if (temp == "")
      break;
    int s;
    file >> s;
    do {
      c = file.get();
    } while (c != '\n');
    if (rank < 0 && s < score)
      rank = count + 1;
    if (worst < 0 || s < worst)
      worst = s;
    scores.push_back(Score(temp, s));
    count++;
  }
  if (rank < 0)
    rank = count;
  if (rank == 0)
    rank = 1;
  file.close();

  if (count < MAX_SCORES-1 || score > worst) {
    char name[15];
    name[0] = '\0';
    int c = 0, t = 0;
    char ascii = 0;
    while (key[KEY_ENTER] || key[KEY_ESC]) {}
    clear_keybuf();
    while (!key[KEY_ENTER] || c == 1) {
      if (key[KEY_ESC] || quitnow)
        exit(0);
      
      clear_to_color(buffer, makecol(255,255,255));
      textprintf_centre_ex(buffer, Font(), SCREEN_W/2, 50, makecol(0,0,0), -1, "High Score #%d / %d!", rank, MAX_SCORES);
      textprintf_centre_ex(buffer, Font(), SCREEN_W/2, 85, makecol(0,0,0), -1, "%s scored %d points", who, score);
      textprintf_centre_ex(buffer, Font(), SCREEN_W/2, SCREEN_H/2, makecol(0,0,0), -1, "Enter your name, %s: %s", who, name);
      blit(buffer, screen, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
      
      while (!keypressed()) {
        if (quitnow)
          exit(0);
      }
      
      int k = readkey();
      
      t++;
      
      if (k >> 8 == KEY_BACKSPACE) {
        if (c > 0)
          c--;
        name[c] = '\0';
      }
      
      ascii = k & 0xff;
      if (c < 14 && ((ascii >= 'A' && ascii <= 'Z') || ascii == ' ' || (ascii >= 'a' && ascii <= 'z') || (ascii >= '0' && ascii <= '9'))) {
        name[c] = ascii;
        name[++c] = '\0';
      }
    }
    
    clear_to_color(buffer, makecol(255,255,255));
    textprintf_centre_ex(buffer, Font(), SCREEN_W/2, 20, makecol(0,0,0), -1, "Best of the Bunch");
    
    ofstream out("scores");
    int count = 0;
    bool outyet = false;
    for (list<Score>::iterator i = scores.begin(); i != scores.end(); ++i) {
      if (!outyet && i->score < score) {
        if (count++ < MAX_SCORES)
          textprintf_centre_ex(buffer, Font(), SCREEN_W/2, 55 + 35 * count, makecol(0,0,0), -1, "%s -- %d", name, score);
        out << name << "\n" << score << endl;
        outyet = true;
      }
      out << i->name << "\n" << i->score << endl;
      if (count++ < MAX_SCORES)
        textprintf_centre_ex(buffer, Font(), SCREEN_W/2, 55 + 35 * count, makecol(0,0,0), -1, "%s -- %d", i->name.c_str(), i->score);
    }
    if (!outyet) {
      out << name << "\n" << score << endl;
      if (count++ < MAX_SCORES)
        textprintf_centre_ex(buffer, Font(), SCREEN_W/2, 55 + 35 * count, makecol(0,0,0), -1, "%s -- %d", name, score);
      outyet = true;
    }
    out.close();
    
    textprintf_centre_ex(buffer, Font(), SCREEN_W/2, 55 + 35 * (MAX_SCORES+2), makecol(0,0,0), -1,  "Press [Enter] to continue");
    
    while (key[KEY_ENTER] || key[KEY_ESC]) {}
    clear_keybuf();
    while (!key[KEY_ENTER]) {
      if (key[KEY_ESC] || quitnow)
        exit(0);
      
      blit(buffer, screen, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
    }
  }
}



